package edu.ucf.cs.courses.cop4610l.beans.streams;

/**
 * @author Mathew B. Lowery
 * @version 1.0
 */

import java.io.*;
import java.util.*;

/**
 * A JavaBean that puts the contents of a <code>InputStream</code> into a
 * <code>String</code> object.  It takes as an argument a stream and returns
 * (via an event) a string.
 */
public class StreamToString implements Serializable {
  /**
   * Listeners of <code>StringConstructedEvent</code>s.
   */
  private Set listeners = new HashSet();
  /**
   * The string created from the input stream.
   */
  private String string;

  public StreamToString() {
  }

  /**
   * There is only a getter method for this property since it is read-only.  It
   * should be called only after hearing a <code>StringConstructedEvent</code>.
   * @return the newly constructed string
   */
  public String getString() {
    return string;
  }

  /**
   * This bean currently can only support one listener.
   * @param listener
   * @throws TooManyListenersException if more than one listener attempts to add
   */
  public void addStringConstructedListener(StringConstructedListener listener)
                                           throws TooManyListenersException {
    synchronized (listeners) {
      // no listeners yet; add it
      if (listeners.isEmpty()) {
        listeners.add(listener);
      }
      else {
        if (listeners.contains(listener)) {
          // OK; just return
          return;
        }
        else {
          // trying to add new listener when another exists already
          throw new TooManyListenersException(
                "only one listener allowed per instance of this class");
        }
      }
    } // sychronized
  }

  public void removeStringConstructedListener(
                                      StringConstructedListener listener) {
    synchronized (listeners) {
      listeners.remove(listener);
    }
  }

  private void fireStringConstructed(StringConstructedEvent e) {
    Iterator iterator;
    synchronized (listeners) {
      iterator = new HashSet(listeners).iterator();
    }
    while (iterator.hasNext()) {
      ((StringConstructedListener) iterator.next()).stringConstructed(e);
    }
  }

  /**
   * Puts the contents of <code>in</code> into a <code>String</code>.  A
   * <code>StringConstructedEvent</code> is fired when done.
   * @param in the input stream whose contents will be put in a string
   */
  public void constructString(InputStream in) {
    StringBuffer buf = new StringBuffer();
    byte[] b = new byte[8];
    try {
      int i = in.read(b);
      while (i != -1) {
        buf.append(new String(b, 0, i));
        i = in.read(b);
      } // while
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    // set property so listeners can read it
    string = buf.toString();
    fireStringConstructed(new StringConstructedEvent(this, buf.toString()));
  }
} // StreamToString